#ifndef POLYPHEMUS_FILE_MODELS_STREETNETWORKINERTPARTICLE_CXX

//////////////
// INCLUDES //

#include "StreetNetworkInertParticle.hxx"
#include "libRa.cxx"
#include "libRs.cxx"

// INCLUDES //
//////////////
  
namespace Polyphemus
{

  template<class T, class ClassChemistry>
  const T StreetNetworkInertParticle<T, ClassChemistry>::pi = acos(-1);

  ////////////////////////////////
  // CONSTRUCTOR AND DESTRUCTOR //
  ////////////////////////////////


  //! Default constructor.
  /*!
    Builds the model. Nothing else is performed.
  */
  template<class T, class ClassChemistry>
  StreetNetworkInertParticle<T, ClassChemistry>::StreetNetworkInertParticle():
    StreetNetworkTransport<T>()
  {
  }


  //! Main constructor.
  /*!
    \param config_file configuration filename.
  */
  template<class T, class ClassChemistry>
  StreetNetworkInertParticle<T, ClassChemistry>::StreetNetworkInertParticle(string config_file):
    StreetNetworkTransport<T>(config_file)
  {
    this->D2_map["StreetSurfaceDepositedMass"] = &StreetSurfaceDepositedMass;
    this->D2_map["StreetResuspension"] = &StreetResuspension;
    this->D2_map["StreetSurfaceDryDepositionRate"] = &StreetSurfaceDryDepositionRate;
    this->D2_map["StreetWashoffRate"] = &StreetWashoffRate;
  }
  

  //! Destructor.
  template<class T, class ClassChemistry>
  StreetNetworkInertParticle<T, ClassChemistry>::~StreetNetworkInertParticle()
  {
    this->ClearStreetVector();
    this->ClearIntersectionVector();
  }

  //! Reads the configuration.
  /*! It reads the description of the domain, the simulation starting-date,
    species lists, options (especially which processes are included) and the
    paths to data input-files.
  */
  template<class T, class ClassChemistry>
  void StreetNetworkInertParticle<T, ClassChemistry>::ReadConfiguration()
  {
    StreetNetworkTransport<T>::ReadConfiguration();

    /*** Options ***/

    this->config.SetSection("[options]");

    this->config.PeekValue("With_deposition",
                           this->option_process["with_deposition"]);
    if (this->option_process["with_deposition"])
      {
	this->config.PeekValue("Collect_dry_flux",
			       this->option_process["collect_total_dry_flux"]);
      }
    this->config.PeekValue("With_scavenging",
                           this->option_process["with_scavenging"]);
    this->config.PeekValue("Scavenging_model", this->scavenging_model);
    this->scavenging_model = lower_case(this->scavenging_model);

    this->config.PeekValue("With_drainage",
			   this->option_process["with_drainage"]);
    this->config.PeekValue("Min_water_drainage",
			   "> 0.0",
			   min_water_drainage); //mm
    if (this->option_process["with_deposition"])
      {
	this->config.PeekValue("Particles_dry_velocity_option",
			       "zhang|giardina|venkatran|muyshondt",
			       particles_dry_velocity_option);
	if(particles_dry_velocity_option == "giardina")
	  this->config.PeekValue("Brownian_diffusion_resistence_option",
				 "paw|chamberlain",
				 brownian_diffusion_resistence_option);
      }

    //resuspension
    this->config.PeekValue("With_resuspension",
                           this->option_process["with_resuspension"]);
    if(this->option_process["with_resuspension"])
      {
	//Precipitation that indicates no resuspension in the streets
	this->config.PeekValue("Max_rain", max_rain);
	
	this->config.PeekValue("f0_hdv",
			       f0_hdv);
	this->config.PeekValue("f0_ldv",
			       f0_ldv);	
      }
    
    this->config.SetSection("[street]");
    if (this->option_process["with_deposition"])
      {
	this->config.PeekValue("Wind_profile",
			       "masson | macdonald ",
			       option_wind_profile);
	this->config.PeekValue("Building_density",
			       this->building_density);
      }
    if(this->option_process["with_resuspension"])
      ReadStreetFlow();

    /*** Species data ***/

    ConfigStream species_data_stream(this->GetSpeciesFile());
    ConfigStream species_stream(this->file_species);
    string species;

    //read diameter in species file
    species_stream.SetSection("[diameter]");
    while (!species_stream.IsEmpty())
      {
	species = species_stream.GetElement();
	species_stream.GetNumber(diameter[species]);
      }

    //mass density in species file
    species_stream.SetSection("[mass_density]");
    while (!species_stream.IsEmpty())
      {
	species = species_stream.GetElement();
	species_stream.GetNumber(mass_density[species]);
      }

    if (this->option_process["with_drainage"])
      {
	species_stream.SetSection("[drainage_efficiency]");
	while (!species_stream.IsEmpty())
	  {
	    species = species_stream.GetElement();
	    species_stream.GetNumber(drainage_efficiency[species]);
	  }
      }
    if (this->scavenging_model == "microphysical")
      {
	// Henry constants in mol / L / atm.
	species_data_stream.SetSection("[henry]");
	while (!species_data_stream.IsEmpty())
	  {
	    species = species_data_stream.GetElement();
	    species_data_stream.GetNumber(henry_constant[species]);
	  }
      }
    // Gas phase diffusivity constants in cm^2 / s.
    species_data_stream.SetSection("[diffusivity]");
    while (!species_data_stream.IsEmpty())
      {
	species = species_data_stream.GetElement();
	species_data_stream.GetNumber(gas_phase_diffusivity[species]);

      }
    CheckConfiguration();
  }

  template<class T, class ClassChemistry>
  void StreetNetworkInertParticle<T, ClassChemistry>::ReadStreetFlow()
  {
    string line;
    vector<string> v;
    vector<string> lines;

    //! Get the input file name for the traffic data.
    this->config.SetSection("[street]");
    this->config.PeekValue("Traffic_flow", file_traffic_flow); 
    this->config.PeekValue("Traffic_speed", file_traffic_speed);

    //reads traffic flow
    ExtStream StreetStream(file_traffic_flow);
    if (!StreetStream.is_open())
      throw string("File ") + file_traffic_flow + " doesn't exist."; 

    Nh_week = 168;
    int Nh_traffic = this->total_nstreet * Nh_week;
    nb_hdv_hour.resize(Nh_traffic);
    nb_ldv_hour.resize(Nh_traffic);
    u_hdv.resize(Nh_week);
    u_ldv.resize(Nh_week);
	
    //StreetStream.GetLine(line);
    for (int i = 0; i < Nh_traffic; ++i)
      {
	StreetStream.GetLine(line);
	v = split(line, ";");
	nb_hdv_hour(i) = to_num<T>(v[3]);
	nb_ldv_hour(i) = to_num<T>(v[4]);  
      }
    StreetStream.Rewind();

    //Reads speed data
    ExtStream StreetSpeed(file_traffic_speed);
    if (!StreetSpeed.is_open())
      throw string("File ") + file_traffic_speed + " doesn't exist.";
	
    StreetSpeed.GetLine(line);
    for (int i = 0; i < Nh_week; ++i)
      {
	StreetSpeed.GetLine(line);
	v = split(line, ";");
	u_hdv(i) = to_num<T>(v[0]);
	u_ldv(i) = to_num<T>(v[1]);
      }
    StreetSpeed.Rewind();	  	
  }
  

  //! Method called at each time step to initialize the model.
  /*!
    \note Empty method.
  */
  template<class T, class ClassChemistry>
  void StreetNetworkInertParticle<T, ClassChemistry>::Init()
  {
    //    StreetNetworkTransport<T>::Init();
    this->SetCurrentDate(this->Date_min);
    InitStreet();
    this->InitIntersection();
    Allocate();
    this->ComputeStreetAngle();
  }

  //! Streets initialization.
  /*!
   */
  template<class T, class ClassChemistry>
  void StreetNetworkInertParticle<T, ClassChemistry>::InitStreet()
  {
    Array<T, 1> init_conc(this->Ns);
    init_conc = 0.0;
    for (int i = 0; i < this->total_nstreet; ++i)
      {
        Street<T>* street = 
          new Street<T>(this->id_street(i), this->begin_inter(i), this->end_inter(i),
                        this->length(i), this->width(i), this->height(i),
                        this->Ns, Nr_photolysis);

        this->StreetVector.push_back(street);
      }
    this->current_street = this->StreetVector.begin();
  }

  //! Checks the configuration.
  /*! 
  */
  template<class T, class ClassChemistry>
  void StreetNetworkInertParticle<T, ClassChemistry>::CheckConfiguration()
  {
    // The configuration-file path is the field "Data_description" in the main
    // configuration file.
    this->config.SetSection("[data]");
    string data_description_file = this->config.PeekValue("Data_description");
    // Opens the configuration file for input data.
    ConfigStream data_description_stream(data_description_file);

    // Photolysis rates.
    if (this->option_process["with_photolysis"])
      this->input_files["photolysis_rates"].ReadFiles(data_description_file,
						      "photolysis_rates");
    else
      this->input_files["photolysis_rates"].Empty();
    for (map<string, string>::iterator i
	   = this->input_files["photolysis_rates"].Begin();
	 i != this->input_files["photolysis_rates"].End(); i++)
      photolysis_reaction_list.push_back(i->first);
    Nr_photolysis = int(photolysis_reaction_list.size());

    if (this->option_process["with_local_data"])
      {
	if(this->option_process["with_deposition"])
	  {
	    if (this->input_files["meteo"]("SurfaceTemperature").empty())
	      throw "Temperature is needed but no input data file was provided.";
	    if (this->input_files["meteo"]("SurfacePressure").empty())
	      throw "Pressure is needed but no input data file was provided.";
	  }
	if(this->option_process["with_scavenging"])
	  {
	    if (this->input_files["meteo"]("SurfaceTemperature").empty())
	      throw "Temperature is needed but no input data file was provided.";
	    if (this->input_files["meteo"]("SurfacePressure").empty())
	      throw "Pressure is needed but no input data file was provided.";
	    if (this->input_files["meteo"]("Rain").empty())
	      throw "Rain is needed but no input data file was provided.";
	  }
      }
    
    if (!this->option_process["with_deposition"]
	&& this->option_process["collect_dry_flux"])
      throw string("Dry deposition fluxes cannot be collected") +
	" without deposition.";

    if (this->option_process["collect_wet_flux"]
	&& (!this->option_process["with_scavenging"]))
      throw string("Wet deposition fluxes cannot be collected") +
	" without scavenging.";
  }

  /////////////////////
  // INITIALIZATIONS //
  /////////////////////


  //! Allocates memory.
  /*! Allocates grids and fields.
   */
  template<class T, class ClassChemistry>
  void StreetNetworkInertParticle<T, ClassChemistry>::Allocate()
  {
    StreetNetworkTransport<T>::Allocate();

    /*** Deposition ***/

    StreetSurfaceDepositedMass.Resize(this->GridS2D, this->GridST2D);
	
    StreetDryDepositionFlux.Resize(this->GridS2D, this->GridST2D);
    WallDryDepositionFlux.Resize(this->GridS2D, this->GridST2D);
    StreetDryDepositionRate.Resize(this->GridS2D, this->GridST2D); // ug/s
    WallDryDepositionRate.Resize(this->GridS2D, this->GridST2D); // ug/s
	
    StreetSurfaceDepositedMass.SetZero();
    StreetDryDepositionFlux.SetZero();
    WallDryDepositionRate.SetZero();
    WallDryDepositionFlux.SetZero();
    StreetDryDepositionRate.SetZero();

    StreetResuspension.Resize(this->GridS2D, this->GridST2D);
    StreetResuspension.SetZero();

    StreetSurfaceDryDepositionRate.Resize(this->GridS2D, this->GridST2D);
    StreetSurfaceDryDepositionRate.SetZero();

    StreetWashoffRate.Resize(this->GridS2D, this->GridST2D);
    StreetWashoffRate.SetZero();
    
    /*** Scavenging ***/
    ScavengingCoefficient.Resize(this->GridS2D, this->GridST2D);
    StreetScavengingFlux.Resize(this->GridS2D, this->GridST2D);
    StreetScavengingRate.Resize(this->GridS2D, this->GridST2D);

    ScavengingCoefficient.SetZero();
    StreetScavengingFlux.SetZero();
    StreetScavengingRate.SetZero();
  }

  //! Performs one step forward.
  template<class T, class ClassChemistry>
  void StreetNetworkInertParticle<T, ClassChemistry>::ComputeScavengingCoefficient()
  {
    int st = 0;
    int s;
    Array<T, 1> coefficient(this->Ns);
    Array<T, 1> level(1);
    level = 2.;
    
    for (typename vector<Street<T>* >::iterator iter = this->StreetVector.begin(); iter != this->StreetVector.end(); iter++)
      {
        Street<T>* street = *iter;
	
        T temperature_ = street->GetTemperature();
        T pressure_ = street->GetPressure();
        T rain_ = street->GetRain();
        T cloudheight_ = street->GetHeight();
        T level_ = street->GetHeight();

	Array<T, 1> Diameter(this->Ns);
	Diameter = 0.0;
	Array<T, 1> Rho(this->Ns);
	Rho = 0.0;
	for (s = 0; s < this->Ns; s++)
	  {
	    Diameter(s) = diameter[this->species_list[s]]; //m
	    Rho(s) = mass_density[this->species_list[s]]; //kg/m3
	  }
	_compute_scavenging_coefficient_aer(&this->Ns,
					    &temperature_,
					    &pressure_,
					    &rain_,
					    Diameter.data(),
					    Rho.data(),
					    &level_,
					    &cloudheight_,
					    coefficient.data());
	
	for (s = 0; s < this->Ns; s++)
	  street->SetStreetScavengingCoefficient(coefficient(s), s);
      }
  }

  template<class T, class ClassChemistry>
  void StreetNetworkInertParticle<T, ClassChemistry>::CalculWashoffFactor()
  {
    int st = 0;
    for (typename vector<Street<T>* >::iterator iter = this->StreetVector.begin();
         iter != this->StreetVector.end(); iter++)
      {
	Street<T>* street = *iter;
	T rain = street->GetRain(); //mm/h
	T groad = rain / 3600. * this->Delta_t; //mm
	T groad_drain_min = min_water_drainage; //mm
	for (int s = 0; s < this->Ns; ++s)
	  {
	    T f_washoff = 0.0;
	    T hdrain_eff = drainage_efficiency[this->species_list[s]];
	    if(groad > groad_drain_min)
	      {
		f_washoff = (1 - exp(-1*hdrain_eff * ((groad - groad_drain_min)/groad_drain_min)))/this->Delta_t; //s-1
		if (f_washoff * 1. != f_washoff)
		  throw string("Error ! NaN in f_washoff");
		f_washoff = max(f_washoff, 0.0);
	      }
	    street->SetStreetWashoffFactor(f_washoff, s);
	  }
	st += 1;
      }    
  }

  template<class T, class ClassChemistry>
  void StreetNetworkInertParticle<T, ClassChemistry>::CalculAerosolResuspensionFactor()
    {
    int ind_day = this->current_date.GetWeekDay();
    int ind_hour = this->current_date.GetHour();
    int ind_line_speed = 24*ind_day + ind_hour;
    T u_ref_susp = 50.0; //km/h
    int st = 0;
    for (typename vector<Street<T>* >::iterator iter = this->StreetVector.begin();
         iter != this->StreetVector.end(); iter++)
      {
	Street<T>* street = *iter;
	int ind_line_flow = Nh_week*st + 24*ind_day + ind_hour;
	T f_resusp = 0.0;
	T rain = street->GetRain();
	if(rain < max_rain)
	  {
	    //HDV
	    T nb_hdv_street = nb_hdv_hour(ind_line_flow); // (veh/h)
	    T u_hdv_street = u_hdv(ind_line_speed); // (veh/h)
	    //LDV
	    T nb_ldv_street = nb_ldv_hour(ind_line_flow);
	    T u_ldv_street = u_ldv(ind_line_speed);
	    T f_resusp_hdv = nb_hdv_street * (u_hdv_street/u_ref_susp) * f0_hdv / 3600.; //s-1
	    T f_resusp_ldv = nb_ldv_street * (u_ldv_street/u_ref_susp) * f0_ldv / 3600.; //s-1
	    f_resusp = f_resusp_hdv + f_resusp_ldv; //s-1
	    if (f_resusp * 1.0 != f_resusp)
	      throw string("Error! f_resusp = NaN");
	  }
	street->SetStreetResuspensionFactor(f_resusp);    
	st ++;
      }
  }
  
  
  //! Performs one step forward.
  template<class T, class ClassChemistry>
  void StreetNetworkInertParticle<T, ClassChemistry>::Transport()
  {
    this->is_stationary = false;
    //! Compute the wind speed in the street-canyon.
    this->ComputeUstreet();
    this->ComputeSigmaW();
    this->ComputeTransferVelocity();
    this->ComputeWindDirectionFluctuation();
    if(this->option_process["with_deposition"])
      ComputeDryDepositionVelocities();
    if(this->option_process["with_scavenging"])
      ComputeScavengingCoefficient();
    if (this->option_process["with_drainage"])
      CalculWashoffFactor();
    if(this->option_process["with_resuspension"])
      CalculAerosolResuspensionFactor();
    this->SetInitialStreetConcentration();
    int niter = 0;
    const int niter_max = this->nintersection;
    while (niter < niter_max and (!this->is_stationary))
      {
	this->InitInflowRate();
	this->ComputeInflowRateExtended();
	//! Compute the concentrations in the street-canyon.
	ComputeStreetConcentration();
	this->IsStationary(this->is_stationary);
	++niter;
      }
    if (!this->is_stationary)
      throw("Error: stationarity is not achieved. Please increase the number of iterations.");
    if (this->option_process["with_deposition"] and this->option_process["with_resuspension"])
      ComputeStreetSurfaceDepositedMass();
  }

  template<class T, class ClassChemistry>
  void StreetNetworkInertParticle<T, ClassChemistry>::ComputeStreetSurfaceDepositedMass()
    {
    int sst = 0;
    for (typename vector<Street<T>* >::iterator iter = this->StreetVector.begin();
         iter != this->StreetVector.end(); iter++)
      {
        Street<T>* street = *iter;
	int id_st = this->id_street(sst);
	T street_area = street->GetWidth() * street->GetLength(); // m2
	T street_volume = street->GetHeight() * street->GetWidth() * street->GetLength(); // m3
	T f_resusp = street->GetStreetResuspensionFactor();
	for (int s = 0; s < this->Ns; ++s)
	  {
	    T f_washoff = street->GetStreetWashoffFactor(s);
	    T f = f_resusp + f_washoff;
	    T street_dry_deposition_rate_aer = street_area * 
	      street->GetStreetDryDepositionVelocity(s); // m3/s
	    T street_scavenging_flux = street_volume *
	      street->GetStreetScavengingCoefficient(s);
	    T street_conc = street->GetStreetConcentration(s);		
	    T street_surface_deposited_mass = street->GetStreetSurfaceDepositedMass(s);
	    if(f > 0.0)
	      {
		T D = (street_dry_deposition_rate_aer + street_scavenging_flux)*street_conc; //mug/s
		T Dmass_diff = street->GetStreetSurfaceDepositedMass(s) - D/f;
		street_surface_deposited_mass = (D/f) + Dmass_diff * exp(-f*this->Delta_t); //mug
	      }
	    street->SetStreetSurfaceDepositedMass(street_surface_deposited_mass, s);
	  }
	sst += 1;
      }
  }
  
  template<class T, class ClassChemistry>
  void StreetNetworkInertParticle<T, ClassChemistry>::ComputeStreetConcentration()
  {
    const T delta_concentration_min = 0.01;

    for (typename vector<Street<T>* >::iterator iter = this->StreetVector.begin();
         iter != this->StreetVector.end(); iter++)
      {
        Street<T>* street = *iter;
        T transfer_velocity = street->GetTransferVelocity(); // m/s
	T temp = transfer_velocity * street->GetWidth() * street->GetLength(); // m3/s
        T outgoing_flux = street->GetOutgoingFlux(); // m3/s
        T street_volume = street->GetHeight() *
          street->GetWidth() * street->GetLength(); // m3
	T street_area = street->GetWidth() * street->GetLength(); // m2
	//! symmetric walls
        T wall_area = 2.0 * street->GetHeight() * street->GetLength(); // m2

	T rain = street->GetRain();
        bool is_stationary_local = true;
        for (int s = 0; s < this->Ns; ++s)
          {
            T street_conc = street->GetStreetConcentration(s); // ug/m3
            T init_street_conc = street->GetInitialStreetConcentration(s); // ug/m3
	    T emission_rate = street->GetEmission(s); //  ug/s
            T inflow_rate = street->GetInflowRate(s); // ug/s
            T conc_bg = street->GetBackgroundConcentration(s); // ug/m3

	    T deposition_rate = 0.0;
	    T scavenging_rate = 0.0;
	    T resuspension_rate = 0.0;
	    T washoff_rate = 0.0;

	    if (this->option_process["with_drainage"])
	      {
		washoff_rate = street->GetStreetWashoffFactor(s) * street->GetStreetSurfaceDepositedMass(s);
	      }
	    
	    if (this->option_process["with_deposition"])
	      {
	    	T street_dry_deposition_rate = street_area * 
	    	  street->GetStreetDryDepositionVelocity(s); // m3/s
	    	T wall_dry_deposition_rate = wall_area * 
	    	  street->GetWallDryDepositionVelocity(s); // m3/s
	    	deposition_rate = street_dry_deposition_rate +
	    	  wall_dry_deposition_rate; // m3/s
		
		if(this->option_process["with_resuspension"])
		  resuspension_rate = street->GetStreetResuspensionFactor() * street->GetStreetSurfaceDepositedMass(s);
	      }

	    if (this->option_process["with_scavenging"])
	      {		
		if (rain > 0.0)
		  scavenging_rate = street_volume * 
		    street->GetStreetScavengingCoefficient(s); // m3/s
	      }

            //! Compute the new concentrations.
            T street_conc_new = 0.0;
	    
            if ((temp + outgoing_flux + deposition_rate + scavenging_rate + washoff_rate) == 0.0)
              {
                //! The total outgoing flux should not be zero
                //! because it leads to an infinite increase of pollutant concentrations
                //! in Munich which uses a steady-state approximation.
                //! In this case, the concentrations are assumed to be constant
                //! because the total incoming flux should be equal to zero.
                street_conc_new = ((emission_rate + resuspension_rate) * this->Delta_t) / street_volume
                  + init_street_conc;
              }
            else
              street_conc_new = (emission_rate + inflow_rate + temp * conc_bg + resuspension_rate) /
                (temp + outgoing_flux + deposition_rate + scavenging_rate + washoff_rate);

            //! Set the minimum possible concentrations.
            street_conc_new = max(street_conc_new, 0.0);

            //! Check the stationarity
            T delta_concentration = abs(street_conc_new - street_conc);
            if ((street_conc != 0.0) and 
                (delta_concentration > delta_concentration_min))
              is_stationary_local = false;

            //! Set the new concentrations.
            street->SetStreetConcentration(street_conc_new, s);
	    T rain = street->GetRain();
	    T street_dry_deposition_rate = street_area * 
	      street->GetStreetDryDepositionVelocity(s); // m3/s
	    
            T massflux_roof_to_background;
            if (is_stationary_local)
              {
                massflux_roof_to_background = temp * (street_conc_new - conc_bg); // ug/s

                street->SetMassfluxRoofToBackground(massflux_roof_to_background, s);

                T conc_delta = street_conc_new - init_street_conc;
                T street_quantity_delta = conc_delta * street_volume; // ug
                street->SetStreetQuantityDelta(street_quantity_delta, s);
              }

	    if (this->option_process["with_deposition"])
	      street->SetStreetDryDepositionFlux(street_conc_new * 
						 street->GetStreetDryDepositionVelocity(s),
						 s); // ug/m2/s
          }

        //! True if stationarity is obtained for a street.
        street->SetStationary(is_stationary_local);

      }
  }
  
    //! Compute dry deposition of gas species for the whole street-network.
  template<class T, class ClassChemistry>
  void StreetNetworkInertParticle<T, ClassChemistry>::ComputeDryDepositionVelocities()
  {
    int st = 0;
    for (typename vector<Street<T>* >::iterator iter = this->StreetVector.begin(); iter != this->StreetVector.end(); iter++)
      {
        Street<T>* street = *iter;
	  
	// Street data
        T H = street->GetHeight();
        T W = street->GetWidth();
        T temperature_ = street->GetTemperature();
        T pressure_ = street->GetPressure();
	
        // Dry deposition
        T Ra_street_recirculation, Ra_wall_recirculation;
	T Ra_street_ventilation, Ra_wall_ventilation;

	Ra_street_recirculation = 0.0;
	Ra_wall_recirculation = 0.0;
	Ra_street_ventilation = 0.0;
	Ra_wall_ventilation = 0.0;
	
        // Building density
        T lambda_p = this->building_density;

        T lambda_f = H / (W + ((lambda_p * W) / (1. - lambda_p)));

        // Displacement height
        T A = 4.0;
        T d = ComputeD(A, lambda_p, H);

        // Mixing length
        T lc = ComputeLc(H, d);

        // Dimensionless parameter in Eq. 22 in Cherin et al. (2015)
        T phi = 0.2;

        T z_can_recirculation = ComputeZcan(W, H);

        T z0_street = 0.01; 
        T z0_wall = 0.0001;

        T zlim = ComputeZlim(lc, phi);

        T WindProfile_coeff = 0.5 * H / W;
  
	if(this->option_wind_profile == "masson")  
          WindProfile_coeff = 0.5 * H / W;
        else if(this->option_wind_profile == "macdonald")
          WindProfile_coeff = 9.6 * lambda_f;

        T Utop = street->GetWindSpeed();
        T Ustar_street = ComputeUstar_Surface(z0_street, zlim, H, WindProfile_coeff, Utop);
        T Ustar_wall = ComputeUstar_Surface(z0_wall, zlim, H, WindProfile_coeff, Utop);
        
        T Dzeta = ComputeUtop_CanyonIntegration(W, H, 3.0 * H);

	T pblh = street->GetPBLH();
	T lmo = street->GetLMO();

        if (lambda_p == 1.0)
          throw string("Building density is irrealistic: 1.0");

        /*** Ra ***/
        // Aerodynamic resistance of the street (Ra)
        Ra_street_recirculation = 
          ComputeRsurface_recirculation_exp_profile(lc, z_can_recirculation, 
                                                    z0_street, H, Utop*Dzeta, 
                                                    WindProfile_coeff, phi);

        // Aerodynamic resistance of the wall (Ra)
        Ra_wall_recirculation = 
          ComputeRsurface_recirculation_exp_profile(lc, z_can_recirculation, 
                                                    z0_wall, H, Utop*Dzeta, 
                                                    WindProfile_coeff, phi);

	if (W > 3*H) //for wide canyons
	  {
	    // Aerodynamic resistance of the street (Ra)
	    Ra_street_ventilation = 
	      ComputeRsurface_ventilated_exp_profile(lc, z_can_recirculation, 
						     z0_street, H, Utop*Dzeta, 
						     WindProfile_coeff, phi);

	    // Aerodynamic resistance of the wall (Ra)
	    Ra_wall_ventilation = 
	      ComputeRsurface_ventilated_exp_profile(lc, z_can_recirculation, 
						     z0_wall, H, Utop*Dzeta, 
						     WindProfile_coeff, phi);
	  }
	
	for(int s = 0; s < this->Ns; s++)
	  {
	    T diameter_ = diameter[this->species_list[s]];
	    T Ts = temperature_;
	    T Ps = pressure_;
	    T ParticleDensity = mass_density[this->species_list[s]];
	    T DynamicViscosity = (pow(Ts,3.)*8.8848e-15) - (pow(Ts,2.)*3.2398e-11)
	      + (pow(Ts,1.)*6.2657e-08) + (pow(Ts,0.)*2.3543e-06);

	    // compute AirDensity = f(Temperature)
	    T AirDensity = 1.293 * (273.15 / Ts);

	    // T ustar = street->GetStreetUstar();
	    T Radius = 10.e-3;
	    T Alpha = 1.5;
	    T Beta = 2.0;
	    T Gamma_street = 0.56;

	    T Rs_street = ComputeSurfaceResistance(DynamicViscosity, Ps, Ts, diameter_, ParticleDensity, AirDensity, Ustar_street, Radius, Alpha, Beta, Gamma_street);

	    T street_dry_deposition_velocity;
	    T vs = ComputeSedimentationVelocity(DynamicViscosity, Ps, Ts,
						diameter_,
						ParticleDensity);

	    if(particles_dry_velocity_option == "muyshondt")
	      {
		T Sc = ComputeSchmidtNumber(DynamicViscosity,
				       Ps, Ts, diameter_,
				       AirDensity);
		T KinematicViscosity = DynamicViscosity / AirDensity;
		T Re_star = Ustar_street*z0_street/KinematicViscosity;
		T b1, b2, b3, b4, b5, b6, tau_p, tau, g;
		b1 = 2.26E-2;
		b2 = 4.03E4;
		b3 = 1.533E4;
		b4 = 0.1394;
		b5 = 49.0;
		b6 = 1.136;
		g = 9.81;
		tau = vs/g;
		tau_p = tau*pow(Ustar_street, 2.)/KinematicViscosity;
		T tmp1 = -0.5*pow((Re_star-b2)/b3, 2.);
		T tmp2 = -0.5*pow((log(tau_p) - log(b5))/b6, 2.);
		T vdi = Ustar_street*(b1*exp(tmp1) - b4*exp(tmp2));
		T vdd = 0.084*pow(Sc,-0.667);
		street_dry_deposition_velocity = vs + vdi + vdd;
		street->SetStreetDryDepositionVelocity(street_dry_deposition_velocity, s);
	      }
	    
	    else if(particles_dry_velocity_option == "venkatran")
	      {
		//T R_street = Ra_street_recirculation + Rs_street;
		T R_street = Ra_street_recirculation + Ra_street_ventilation + Rs_street;
		if (R_street <= 0.0)
		  throw string("Error in deposistion, R_street: ")
		    + to_str(R_street);

		
		street_dry_deposition_velocity = vs / (1. - exp(-vs * R_street));
	    
		street->SetStreetDryDepositionVelocity(street_dry_deposition_velocity, s);
	      }

	    else if(particles_dry_velocity_option == "zhang")
	      {
		T R_street = Ra_street_recirculation + Ra_street_ventilation + Rs_street;
		if (R_street <= 0.0)
		  throw string("Error in deposistion, R_street: ")
		    + to_str(R_street);

		
		street_dry_deposition_velocity = vs + 1/(R_street);
	    
		street->SetStreetDryDepositionVelocity(street_dry_deposition_velocity, s);
	      }

	    else if(particles_dry_velocity_option == "giardina")
	      {
		//Rbd: Brownian diffusion process
		//Rii: inertial impact conditions
		//Rti: turbulent impactation
		T Sc = ComputeSchmidtNumber(DynamicViscosity,
					    Ps, Ts, diameter_,
					    AirDensity);
		T Rbd, Rii, Rti, R, St_rought, b;
		T KinematicViscosity = DynamicViscosity / AirDensity;
		T tau = vs/g;
		T tau_p = tau*pow(Ustar_street, 2.)/KinematicViscosity;
		T Re_star = Ustar_street*z0_street/KinematicViscosity;
		St_rought = ComputeStokesNumberRough(DynamicViscosity, Ps, Ts, diameter_, ParticleDensity, AirDensity, Ustar_street, Radius);
		b = 2.0;
		R = exp(-b*sqrt(St_rought));
		T tmp = pow(St_rought, 2.)/(pow(St_rought, 2.) + 1.);
		Rii = 1./(Ustar_street*tmp*R);
		T m, n;
		m = 0.05;
		n = 0.75;
		Rti = 1./(Ustar_street*m*pow(tau_p, n)*R);
		if (brownian_diffusion_resistence_option == "paw")
		  Rbd = 1./(Ustar_street*pow(Sc, (-2./3.)));
		else if(brownian_diffusion_resistence_option == "chamberlain")
		  Rbd = 1./(Ustar_street*pow(Sc, -0.5)*pow(Re_star, -0.05));
		else
		  throw string("Error in brownian_diffusion_resistence_option");
		T Req = 1./Rbd + 1./(Rii + Rti);
		T Ra_street = Ra_street_recirculation + Ra_street_ventilation;
		street_dry_deposition_velocity = vs/(1-exp(-vs*(Ra_street + 1./Req)));
		street->SetStreetDryDepositionVelocity(street_dry_deposition_velocity, s);
	      }
	      
	    T Rs_wall = ComputeSurfaceResistance(DynamicViscosity, Ps, Ts, diameter_, ParticleDensity, AirDensity, Ustar_wall, Radius, Alpha, Beta, Gamma_street);
	    T R_wall = Ra_wall_recirculation + Ra_wall_ventilation + Rs_wall;
	    if (R_wall <= 0.0)
	      throw string("Error in deposistion, R_wall: ")
		+ to_str(R_wall);
	    T wall_dry_deposition_velocity = 1. / R_wall;
	    
	    street->SetWallDryDepositionVelocity(wall_dry_deposition_velocity, s);

	  }
	st += 1;
      }
  }
    

  //! Performs one step forward.
  template<class T, class ClassChemistry>
  void StreetNetworkInertParticle<T, ClassChemistry>::Forward()
  {
    Transport();
    this->SetStreetConcentration();
    SetStreetSurfaceDepositedMass();
    SetStreetResuspension();
    SetStreetSurfaceDryDepositionRate();
    SetStreetWashoffRate();
    this->SetStreetConcentration();
    this->AddTime(this->Delta_t);
    this->step++;
  }

  //! Method called at each time step to initialize the model.
  /*!
    \note Empty method.
  */
  template<class T, class ClassChemistry>
  void StreetNetworkInertParticle<T, ClassChemistry>::InitData()
  {
    StreetNetworkTransport<T>::InitData();

    /*** Meteo for the chemistry on the streets ***/
    string filename;
    if (this->option_process["with_local_data"])
      {
        attenuation.resize(this->Nt_meteo, this->total_nstreet);
        filename = this->input_files["meteo"]("Attenuation");
        InitData(filename, attenuation);

        specific_humidity.resize(this->Nt_meteo, this->total_nstreet);
        filename = this->input_files["meteo"]("SpecificHumidity");
        InitData(filename, specific_humidity);

        pressure.resize(this->Nt_meteo, this->total_nstreet);
        filename = this->input_files["meteo"]("SurfacePressure");
        InitData(filename, pressure);

        temperature.resize(this->Nt_meteo, this->total_nstreet);
        filename = this->input_files["meteo"]("SurfaceTemperature");
        InitData(filename, temperature);

	rain.resize(this->Nt_meteo, this->total_nstreet);
        filename = this->input_files["meteo"]("Rain");
        InitData(filename, rain);
      }

  }

  //! Method called at each time step to initialize the model.
  /*!
    \note Empty method.
  */
  template<class T, class ClassChemistry>
  void StreetNetworkInertParticle<T, ClassChemistry>::InitData(string input_file, 
                                                           Array<T, 2>& input_data)
  {
    StreetNetworkTransport<T>::InitData(input_file, input_data);
  }

  //! Method called at each time step to initialize the model.
  /*!
    \note Empty method.
  */
  template<class T, class ClassChemistry>
  void StreetNetworkInertParticle<T, ClassChemistry>::InitStep()
  {
    StreetNetworkTransport<T>::InitStep();

    int st = 0;
    for (typename vector<Street<T>* >::iterator iter = this->StreetVector.begin();
         iter != this->StreetVector.end(); iter++)
      {
        Street<T>* street = *iter;

        if (this->option_process["with_local_data"])
          {
            //! Set the meteo data
            street->SetMeteoInertParticle(attenuation(this->meteo_index, st),
					  pressure(this->meteo_index, st),
					  specific_humidity(this->meteo_index, st),
					  temperature(this->meteo_index, st),
					  rain(this->meteo_index, st)); //LL
          }
      }

  }

  template<class T, class ClassChemistry>
  void StreetNetworkInertParticle<T, ClassChemistry>
  ::SetStreetAdditionalMeteo(int street_index,
			     T attenuation,
                             T pressure,
			     T specific_humidity,
                             T temperature,
			     T rain)
  {
    int st = 0;
    for (typename vector<Street<T>* >::iterator iter = this->StreetVector.begin();
         iter != this->StreetVector.end(); iter++)
      {
        if (st == street_index)
          {
            Street<T>* street = *iter;
            street->SetMeteoInertParticle(attenuation,
					  pressure,
					  specific_humidity, 
					  temperature,
					  rain);
          }
        ++st;
      }

  }


  //! Returns the resuspension rate for the whole street-network.
  template<class T, class ClassChemistry>
  inline Data<T, 2>& StreetNetworkInertParticle<T, ClassChemistry>::GetStreetResuspension()
  {
    return StreetResuspension;
  }

  
  //! Returns the deposited mass for the whole street-network.
  template<class T, class ClassChemistry>
  inline Data<T, 2>& StreetNetworkInertParticle<T, ClassChemistry>::GetStreetSurfaceDepositedMass()
  {
    return StreetSurfaceDepositedMass;
  }

  
  //! Returns the deposition rate for the whole street-network.
  template<class T, class ClassChemistry>
  inline Data<T, 2>& StreetNetworkInertParticle<T, ClassChemistry>::GetStreetSurfaceDryDepositionRate()
  {
    return StreetSurfaceDryDepositionRate;
  }

  
  //! Returns the washoff rate for the whole street-network.
  template<class T, class ClassChemistry>
  inline Data<T, 2>& StreetNetworkInertParticle<T, ClassChemistry>::GetStreetWashoffRate()
  {
    return StreetWashoffRate;
  }

  //! Sets the surface deposited mass for the whole street-network.
  template<class T, class ClassChemistry>
  void StreetNetworkInertParticle<T, ClassChemistry>::SetStreetSurfaceDepositedMass()
  {
    int ist = 0;
    for (typename vector<Street<T>* >::iterator iter = this->StreetVector.begin();
         iter != this->StreetVector.end(); iter++)
      {
        Street<T>* street = *iter;
        for (int s = 0; s < this->Ns; ++s)
          StreetSurfaceDepositedMass(s, ist) = street->GetStreetSurfaceDepositedMass(s);
        ++ist;
      }
  }

  //! Sets the resuspension rate for the whole street-network.
  template<class T, class ClassChemistry>
  void StreetNetworkInertParticle<T, ClassChemistry>::SetStreetResuspension()
  {
    int ist = 0;
    for (typename vector<Street<T>* >::iterator iter = this->StreetVector.begin();
         iter != this->StreetVector.end(); iter++)
      {
        Street<T>* street = *iter;
        for (int s = 0; s < this->Ns; ++s)
          StreetResuspension(s, ist) = street->GetStreetResuspensionFactor() * street->GetStreetSurfaceDepositedMass(s);
        ++ist;
      }
  }

  //! Sets the washoff rate for the whole street-network.
  template<class T, class ClassChemistry>
  void StreetNetworkInertParticle<T, ClassChemistry>::SetStreetWashoffRate()
  {
    int ist = 0;
    for (typename vector<Street<T>* >::iterator iter = this->StreetVector.begin();
         iter != this->StreetVector.end(); iter++)
      {
        Street<T>* street = *iter;
        for (int s = 0; s < this->Ns; ++s)
	  {
	    StreetWashoffRate(s, ist) = street->GetStreetWashoffFactor(s) * street->GetStreetSurfaceDepositedMass(s);
	  }
        ++ist;
      }
  }

  //! Sets the surface deposited mass for the whole street-network.
  template<class T, class ClassChemistry>
  void StreetNetworkInertParticle<T, ClassChemistry>::SetStreetSurfaceDryDepositionRate()
  {
    int ist = 0;
    for (typename vector<Street<T>* >::iterator iter = this->StreetVector.begin();
         iter != this->StreetVector.end(); iter++)
      {
        Street<T>* street = *iter;	
	T street_area = street->GetWidth() * street->GetLength(); // m2
      
        for (int s = 0; s < this->Ns; ++s)
	  StreetSurfaceDryDepositionRate(s, ist) = street_area * street->GetStreetDryDepositionVelocity(s) * street->GetStreetConcentration(s);

        ++ist;
      }
  }
} // namespace Polyphemus.


#define POLYPHEMUS_FILE_MODELS_STREETNETWORKINERTPARTICLE_CXX
#endif
