#ifndef POLYPHEMUS_FILE_MODELS_STREETNETWORKINERTPARTICLE_HXX

#include "StreetNetworkTransport.cxx"
#include "StreetTransport.cxx"

namespace Polyphemus
{


  using namespace std;
  using namespace blitz;

  //////////////////////
  // FORTRAN FUNCTION //
  //////////////////////


#ifdef POLYPHEMUS_SINGLE_UNDERSCORE
#undef POLYPHEMUS_DOUBLE_UNDERSCORE
#elif defined(__GNUG__) && __GNUG__ < 4 && !defined(__INTEL_COMPILER)
#undef POLYPHEMUS_DOUBLE_UNDERSCORE
#define POLYPHEMUS_DOUBLE_UNDERSCORE
#endif

#ifdef POLYPHEMUS_DOUBLE_UNDERSCORE
  //#define _compute_dry_deposition compute_dry_deposition__
#define _compute_scavenging_coefficient_aer	\
  compute_scavenging_coefficient_aer__
#else
  //#define _compute_dry_deposition compute_dry_deposition_
#define _compute_scavenging_coefficient_aer	\
  compute_scavenging_coefficient_aer_
#endif

  extern "C"
  {
    void _compute_scavenging_coefficient_aer(int*, double*, double*, double*,
					     double*, double*, double*,
					     double*, double*);
  }  


  //////////////////
  // STREET //
  //////////////////


  //! StreetNetworkInertParticle model.
  template<class T, class ClassChemistry>
  class StreetNetworkInertParticle: public StreetNetworkTransport<T>
  {

  protected:

    static const T pi;

    /*** Meteorological data ***/
    Array<T, 2> attenuation;
    Array<T, 2> specific_humidity;
    Array<T, 2> pressure;
    Array<T, 2> temperature;
    Array<T, 2> rain; 

    //***** Chemistry *****//

    ClassChemistry Chemistry_;

    //*** Photolysis ***//
    //! Number of photolysis reactions.
    int Nr_photolysis;
    //! List of species with photolysis reactions.
    vector<string> photolysis_reaction_list;

    /*** New variables to particles ***/
    T min_water_drainage;
    string particles_dry_velocity_option;
    string brownian_diffusion_resistence_option;
    T max_rain;
    T f0_hdv; //s-1
    T f0_ldv; //s-1
    string file_traffic_flow;
    string file_traffic_speed;
    Array<T, 1> nb_hdv_hour;
    Array<T, 1> nb_ldv_hour;
    Array<T, 1> u_hdv;
    Array<T, 1> u_ldv;
    int Nh_week;
    //! diameter for each species.
    map<string, T> diameter;
    //! mass density of each species.
    map<string, T> mass_density;
    //! Drainage efficiency coefficient factor.
    map<string, T> drainage_efficiency;
    
    /*** Deposited mass over streets ***/
    Data<T, 2> StreetSurfaceDepositedMass;
    /*** Deposited mass over streets ***/
    Data<T, 2> StreetResuspension;
    /*** Deposited mass over streets ***/
    Data<T, 2> StreetSurfaceDryDepositionRate;
    /*** Deposited mass over streets ***/
    Data<T, 2> StreetWashoffRate;

    //! Dry deposition fluxes at current date.
    Data<T, 2> StreetDryDepositionFlux;
    //! Dry wall deposition fluxes at current date.
    Data<T, 2> WallDryDepositionFlux;
    //! Dry deposition rate at current date.
    Data<T, 2> StreetDryDepositionRate;
    //! Dry wall deposition rate at current date.
    Data<T, 2> WallDryDepositionRate;

    //! Scavenging coefficient at current date.
    Data<T, 2> ScavengingCoefficient;
    //! Scavenging fluxes at current date.
    Data<T, 2> StreetScavengingFlux;
    //! Scavenging rate at current date.
    Data<T, 2> StreetScavengingRate;
    //!  Scavenging rain threshold (mm / h).
    T scavenging_rain_threshold;
    
    string option_wind_profile;

    /*** Species ***/

    //! Henry constants (mol / L / atm).
    map<string, T> henry_constant;
    //! Gas phase diffusivities (cm^2 / s).
    map<string, T> gas_phase_diffusivity;
    //! Constant scavenging coefficient (s^{-1}).
    map<string, T> scavenging_constant;

    //! Alpha scaling factor.
    map<string, T> alpha;
    //! Beta scaling factor..
    map<string, T> beta;
    

  public:

    /*** Constructors and destructor ***/

    StreetNetworkInertParticle();
    StreetNetworkInertParticle(string config_file);
    virtual ~StreetNetworkInertParticle();

    /*** Initializations ***/

    void ReadConfiguration();
    void CheckConfiguration();
    void Allocate();    
    void Init();
    void InitStreet();
    void InitStep();
    void InitData();
    void InitData(string input_file, Array<T, 2>& input_data);

    /*** Access Methods ***/

    void SetStreetAdditionalMeteo(int street_index,
				  T attenuation,
				  T pressure,
                                  T specific_humidity,
                                  T temperature,
				  T rain);

    /*** Computational Methods ***/
    
    void Forward();

    Data<T, 2>& GetStreetSurfaceDepositedMass();
    Data<T, 2>& GetStreetResuspension();
    Data<T, 2>& GetStreetSurfaceDryDepositionRate();
    Data<T, 2>& GetStreetWashoffRate();
    void SetStreetSurfaceDepositedMass();
    void SetStreetResuspension();
    void SetStreetSurfaceDryDepositionRate();
    void SetStreetWashoffRate();
    void Transport();
    void ComputeDryDepositionVelocities();
    void ComputeScavengingCoefficient();
    void CalculWashoffFactor();
    void CalculAerosolResuspensionFactor();
    void ReadStreetFlow();
    void ComputeStreetConcentration();
    void ComputeStreetSurfaceDepositedMass();
  };


} // namespace Polyphemus.


#define POLYPHEMUS_FILE_MODELS_STREETNETWORKINERTPARTICLE_HXX
#endif
